@@ -22,7 +22,12 @@ from datetime import datetime, timedelta
 from mrack.context import global_context
 from mrack.errors import ProvisioningError
 from mrack.host import STATUS_ACTIVE, STATUS_OTHER, Host
-from mrack.utils import get_username_pass_and_ssh_key, object2json, ssh_to_host
+from mrack.utils import (
+    get_ssh_options,
+    get_username_pass_and_ssh_key,
+    object2json,
+    ssh_to_host,
+)
 
 logger = logging.getLogger(__name__)
 
@@ -128,12 +133,16 @@ class Provider:
         )
 
         while True:
+            ssh_options = get_ssh_options(
+                host, global_context.METADATA, global_context.PROV_CONFIG
+            )
             res = ssh_to_host(
                 host,
                 username=username,
                 password=password,
                 ssh_key=ssh_key,
                 command="echo mrack",
+                ssh_options=ssh_options,
             )
             duration = (datetime.now() - start_ssh).total_seconds()
 
@@ -415,6 +424,10 @@ class Provider:
             req for req in reqs if req["name"] in [host.name for host in error_hosts]
         ]
 
+    def _get_max_utilization(self):
+        """Get max utilization value from context."""
+        return global_context.CONFIG.max_utilization
+
     async def strategy_retry(self, reqs):
         """Provisioning strategy to try multiple times to provision a host."""
         log_msg_start = self.dsp_name
@@ -423,7 +436,7 @@ class Provider:
         success_hosts = []
         error_hosts = []
         delta_sleep = global_context.CONFIG.delta_sleep
-        max_utilization = global_context.CONFIG.max_utilization
+        max_utilization = self._get_max_utilization()
 
         while missing_reqs:
             # number of attempts should be greater than max_retry
@@ -450,8 +463,14 @@ class Provider:
             s_hosts, error_hosts, missing_reqs = await self._provision_base(
                 missing_reqs, timeout=res_check_timeout
             )
+
             success_hosts.extend(s_hosts)
 
+            if s_hosts:
+                logger.info(f"{log_msg_start} {len(s_hosts)} host(s) provisioned.")
+                succ = f"{len(success_hosts)}/{len(reqs)} host(s) provisioned properly."
+                logger.info(f"{log_msg_start} Provisioning progress: {succ}")
+
             # We need to cleanup the error hosts for the next retry run
             # in case of last attempt which is `self.max_retry`
             # we skip this part at it done in provision_hosts() method while
@@ -459,40 +478,45 @@ class Provider:
 
             if error_hosts and attempts <= self.max_retry:
                 count = len(error_hosts)
-                err = f"{count} hosts were not provisioned properly, deleting."
+                err = f"{count} host(s) NOT provisioned properly, deleting."
                 logger.info(f"{log_msg_start} {err}")
-                out_of_resources = False
+                server_error = False
                 for host in error_hosts:
                     logger.error(f"{log_msg_start} Error: {str(host.error)}")
                     # if host.error is a dictionary and code is 500 = SERVER ERROR
                     if (
                         isinstance(host.error, dict)
                         and host.error.get("code", None) == 500
+                        and not server_error  # add condition here to not repeat message
                     ):
                         logger.info(
-                            f"{log_msg_start} Provider is out of resources, "
+                            f"{log_msg_start} Provider returned server error (500), "
                             "increasing cooldown time before another retry"
                         )
-                        out_of_resources = True
+                        server_error = True
 
-                if await self.utilization() >= max_utilization or out_of_resources:
+                error_hosts_share = (
+                    len(error_hosts) / len(success_hosts + error_hosts) * 100
+                )
+                high_utilization = await self.utilization() >= max_utilization
+
+                if high_utilization or (error_hosts_share > max_utilization):
                     # delete all hosts when there is high utilization of provider
+                    # or error hosts share is exceeding the same utilization threshold
                     error_hosts += success_hosts
                     missing_reqs = self._get_missing_reqs(reqs, error_hosts)
                     success_hosts = []
 
                 await self.delete_hosts(error_hosts)
 
-                # multiply the sleep time to increase the cooldown period
-                # to get range from 15 to 30 minutes
-                # before another retry (using default delta_sleep=15)
-                res_check_timeout += (
-                    (delta_sleep - 1) * res_check_timeout * int(out_of_resources)
-                )  # -1 because we do addition to original value and multiply by 14
+                # increase the sleep time to extend the cooldown period
+                # when internal error happens
+                res_check_timeout += res_check_timeout * int(server_error)
 
                 cooldown = random.choice(
                     range(res_check_timeout, 2 * res_check_timeout)
                 )
+
                 logger.info(
                     f"{log_msg_start} Retrying to provision these hosts in {cooldown}s"
                 )
